==> pymimir-0.12.13.tar.gz: pymimir-0.12.13/include/mimir/search/openlists/interface.hpp <==
/*
 * Copyright (C) 2023 Dominik Drexler and Simon Stahlberg
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef MIMIR_SEARCH_OPENLISTS_INTERFACE_HPP_
#define MIMIR_SEARCH_OPENLISTS_INTERFACE_HPP_

#include <concepts>
#include <cstdint>

namespace mimir::search
{

/**
 * Interface class
 */
template<typename Derived_>
class IOpenList
{
private:
    IOpenList() = default;
    friend Derived_;

    /// @brief Helper to cast to Derived_.
    constexpr const auto& self() const { return static_cast<const Derived_&>(*this); }
    constexpr auto& self() { return static_cast<Derived_&>(*this); }

public:
    template<typename T>
    void insert(double priority, const T& item)
    {
        return self().insert_impl(priority, item);
    }

    decltype(auto) top() const { return self().top_impl(); }

    void pop() { self().pop_impl(); }

    void clear() { self().clear_impl(); }

    bool empty() const { return self().empty_impl(); }
    std::size_t size() const { return self().size_impl(); }
};

}

#endif

==> pymimir-0.13.19.tar.gz: pymimir-0.13.19/include/mimir/search/openlists/interface.hpp <==
/*
 * Copyright (C) 2023 Dominik Drexler and Simon Stahlberg
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef MIMIR_SEARCH_OPENLISTS_INTERFACE_HPP_
#define MIMIR_SEARCH_OPENLISTS_INTERFACE_HPP_

#include <concepts>
#include <cstdint>
#include <utility>

namespace mimir::search
{

/**
 * Interface class
 */
template<typename Derived_>
class IOpenList
{
private:
    IOpenList() = default;
    friend Derived_;

    /// @brief Helper to cast to Derived_.
    constexpr const auto& self() const { return static_cast<const Derived_&>(*this); }
    constexpr auto& self() { return static_cast<Derived_&>(*this); }

public:
    template<std::totally_ordered Key, typename T>
    void insert(Key&& priority, T&& item)
    {
        return self().insert_impl(std::forward<Key>(priority), std::forward<T>(item));
    }

    decltype(auto) top() const { return self().top_impl(); }

    void pop() { self().pop_impl(); }

    void clear() { self().clear_impl(); }

    bool empty() const { return self().empty_impl(); }
    std::size_t size() const { return self().size_impl(); }
};

}

#endif
